import {Component, OnInit} from '@angular/core';
import { MovieService } from '../../services/movie.service';
import { ActorService } from '../../services/actor.service';
import { Movie } from '../../model/movie.model';
import {Actor} from "../../model/actor.model";
import {MatSnackBar} from "@angular/material/snack-bar";
import {UserService} from "../login-page/user.service";

@Component({
  selector: 'app-admin-page',
  templateUrl: './admin-page.component.html',
  styleUrls: ['./admin-page.component.css']
})


export class AdminPageComponent implements OnInit {
  displayedColumns: string[] = ['posterImage', 'title', 'releaseYear', 'genre', 'director', 'duration', 'language',  'actors', 'actions'];

  newMovie: Movie = {
    title: '',
    userId: '',
    genre: '',
    releaseYear: 0,
    director: '',
    duration: 0,
    language: '',
    description: '',
    posterImage: "",
    actorIds: []
  };

  selectedFile: string | ArrayBuffer | null = null;
  movies: Movie[] = [];
  actors: Actor[] = [];

  constructor(private movieService: MovieService, private actorService: ActorService, private snackBar: MatSnackBar, private userService: UserService,) { }

  ngOnInit(): void {
    this.loadMovies();
    this.loadActors();
  }

  loadMovies(): void {
    this.movieService.getAllMovies().subscribe(movies => {
      this.movies = movies;
    });
  }

  loadActors(): void {
    this.actorService.getAllActors().subscribe(actors => {
      this.actors = actors;
    });
  }


  onFileSelected(event: any) {
    const myReader = new FileReader();
    myReader.onloadend = () => {
      this.selectedFile = myReader.result
    }
    myReader.readAsDataURL(event.target.files[0]);
  }

  getActorNames(actorIds: []): string {

    return actorIds.map(id => {
      const actor = this.actors.find(a => a.actorId === id);
      return actor ? actor.name : 'Unknown';
    }).join(', ');
  }

  createMovie() {
    if (this.selectedFile) {
      this.newMovie.posterImage = this.selectedFile
      this.newMovie.userId = <string>this.userService.getUserId();

      this.movieService.createMovie(this.newMovie).subscribe(response => {
        console.log('Movie created successfully', response);
        this.loadMovies();
        this.snackBar.open('User created successfully!', 'Close', {
          duration: 3000,
          panelClass: ['custom-snack-bar'],
        });
      }, error => {
        console.error('Error creating movie', error);
          this.snackBar.open('Server error.', 'Close', {
          duration: 3000,
          panelClass: ['custom-snack-bar-error'],
        });
      });
    } else {
      console.error('No file selected');
      this.snackBar.open('No file selected', 'Close', {
        duration: 3000,
        panelClass: ['custom-snack-bar-error'],
      });
    }
  }

  editMovie(movie: Movie): void {
    this.newMovie = { ...movie }; // Populate form with the existing movie data
  }

  updateMovie(): void {
    this.newMovie.userId = <string>this.userService.getUserId();

    this.movieService.updateMovie(this.newMovie).subscribe(
      () => {
        this.loadMovies();
        this.resetMovieForm();
        this.snackBar.open('Movie updated successfully!', 'Close', {
          duration: 3000,
          panelClass: ['custom-snack-bar'],
        });
      },
      error => {
        console.error('Error updating movie', error);
        this.snackBar.open('Server error.', 'Close', {
          duration: 3000,
          panelClass: ['custom-snack-bar-error'],
        });
      }
    );
  }

  deleteMovie(movieId: string): void {
    this.movieService.deleteMovie(movieId, this.userService.getUserId()!).subscribe(
      () => {
        this.loadMovies();
        this.snackBar.open('Movie deleted successfully!', 'Close', {
          duration: 3000,
          panelClass: ['custom-snack-bar'],
        });
      },
      error => {
        console.error('Error deleting movie', error);
        this.snackBar.open('Server error.', 'Close', {
          duration: 3000,
          panelClass: ['custom-snack-bar-error'],
        });
      }
    );
  }


  resetMovieForm(): void {
    this.newMovie = {
      posterImage: '',
      userId:'',
      title: '',
      genre: '',
      releaseYear: 2023,
      director: '',
      duration: 0,
      language: '',
      description: '',
      actorIds: []
    };
  }
}
