import { Injectable } from '@angular/core';
import {HttpClient, HttpErrorResponse} from '@angular/common/http';
import {BehaviorSubject, Observable, tap} from 'rxjs';
import { User } from './user.model';
import {catchError, throwError} from 'rxjs';

@Injectable({
  providedIn: 'root'
})
export class UserService {
  private apiUrl = 'http://localhost:8080/api/users';
  username: string | null = null;
  userRole: string | null = null;
  userIcon: string = '../../assets/images/';


  constructor(private http: HttpClient) { }

  createUser(user: User): Observable<User> {
    console.log(user)
    return this.http.post<User>(this.apiUrl, user);
  }

  getUserImg(): String {
    return this.userIcon;
  }

  getUserById(userId: string): Observable<User> {
    return this.http.get<User>(`${this.apiUrl}/${userId}`);
  }

  getAllUsers(): Observable<User[]> {
    return this.http.get<User[]>(this.apiUrl);
  }

  deleteUserById(userId: string): Observable<void> {
    return this.http.delete<void>(`${this.apiUrl}/${userId}`);
  }


  // Login

  private usernameKey = 'username';
  private userIdKey = 'userId';
  private userRoleKey = 'role';

  private usernameSubject = new BehaviorSubject<string | null>(this.getUserName());
  username$ = this.usernameSubject.asObservable();

  private userRoleSubject = new BehaviorSubject<string | null>(this.getUserRole());
  userRole$ = this.userRoleSubject.asObservable();


  loginRequest(email: string, password: string): Observable<User> {
    return this.http.post<User>(`${this.apiUrl}/login`, { email, password }).pipe(
      tap((user: User) => this.loginUserStatusUpdate(user)),

      catchError((error: HttpErrorResponse) => {
        console.error('Error logging in:', error.message);
        return throwError(error.error || 'Server error');
      })
    );
  }

  loginUserStatusUpdate(user: User) {
    this.setUserName(user.username);
    this.setUserRole(user.role);
    this.setUserId(user.userId!) // Will Be not null

    this.usernameSubject.next(user.username);
    this.userRoleSubject.next(user.role);
  }

  logoutUserStatusUpdate(): void {
    sessionStorage.removeItem(this.usernameKey);
    sessionStorage.removeItem(this.userIdKey);
    sessionStorage.removeItem(this.userRoleKey);
    this.usernameSubject.next(null);
    this.userRoleSubject.next(null);
  }


  // Session Storage


  getUserId(): string | null {
    const userData = sessionStorage.getItem('userId');
    if (userData) {
      return userData;
    }
    return null;
  }

  setUserId(userId: string): void {
    sessionStorage.setItem(this.userIdKey, userId);
  }

  getUserName(): string | null {
    return sessionStorage.getItem(this.usernameKey);
  }
  setUserName(username: string): void {
    sessionStorage.setItem(this.usernameKey, username);
  }

  getUserRole(): string | null {
    return sessionStorage.getItem(this.userRoleKey);
  }
  setUserRole(userRole: string): void {
    sessionStorage.setItem(this.userRoleKey, userRole);
  }

}
